#!/usr/bin/perl
# TextAreaWiki (http://ta2o.net/tools/textareawiki/)
# -- The Association of Old-Type Perl Programming (AOTPP)
#
# setup
# 1. Put "taw.cgi" and "data.txt" in a web directory.
# 2. Change $password in "taw.cgi"
# 3. chmod a+x taw.cgi; chmod a+w data.txt
# 4. Access taw.cgi through a web brawser
#
use strict;
use warnings;
use CGI;
use HTML::Template;

### vvvvvv
my $datafile = "data.txt";
my $password = "";
### ^^^^^^

my $q = new CGI;
my $url = $q->url();
my $mode = $q->param('mode') || "";
my $input_password = $q->param('input_password');

if ($password) {
    if (not defined $input_password) {
	$input_password = $q->cookie('password') || "";
    }
    my $cookie = $q->cookie(-name => 'password',
			    -value => $input_password,
			    -path => $q->url(-absolute => 1),
			    -domain => $q->server_name(),
	);
    print $q->header(-charset => 'UTF-8', -cookie => $cookie);
} else {
    print $q->header(-charset => 'UTF-8');
}

if ($mode eq 'write') {
    my $text = $q->param('text');
    my $message = "";
    if ($password and $password ne $input_password) {
        $message = "fail: bad password";
    } elsif ($text =~ m!</textarea.*$!gsmi) {
        $message = "fail: unsafe text";
    } elsif ($text =~ /^\s*$/) {
        $message = "fail: empty text";
    } elsif  (not $q->referer() or $url eq substr($q->referer(), length($url))) {
	$message = "fail: no/bad referer (CRSF)";
    } else {
        save_file($datafile, $text);
        $message = "saved";
    }
    print qq(<html><head><META HTTP-EQUIV="Refresh" ).
        qq(CONTENT="1;URL=$url"></head><body>$message</body></html>\n);
    exit;
}

my $cont = read_file($datafile);
my $line_num = int(@{[$cont =~ m/\n/g]} * 1.1 + 2); # ad hoc

my ($title) = ($cont =~ /^\s*(\S.+?)\n/sm);
$title = "TextAreaWiki" unless $title;

my $template = join("", <DATA>);
my $t = HTML::Template->new(scalarref => \$template,
			    die_on_bad_params => 0);
$t->param(cont => $cont);
$t->param(title => $title);
$t->param(url => $url);
$t->param(line_num => $line_num);
$t->param(password => $password);
$t->param(input_password => $input_password);

print $t->output();

sub read_file {
    my ($fn) = @_;
    if (open(my $fh, "<", $fn)) {
        my $c = join('', (<$fh>));
        close $fh;
        return $c;
    } else {
        return "file open error";
    }
}

sub save_file {
    my ($fn, $str) = @_;
    if (open(my $fh, ">",  $fn)) {
        binmode($fh);
        flock($fh, 2);
        $str =~ s/\r//g;
        print $fh $str;
        close $fh;
        chmod 0666, $fn;
    }
}

__DATA__
<!DOCTYPE HTML>
<html>
<head>
<meta charset="utf-8">
<meta name="viewport" content="width=device-width, initial-scale=1">
<title><TMPL_VAR name=title></title>
<style type="text/css">
html,body {overflow: hidden;}
body {position: fixed; width: 100%; height:100%; margin:0; padding:0;}
form {margin:0;padding:0;height:100%;}
.edit {width:100%;height:100%;margin:0;padding:0.5em;box-sizing:border-box;resize:none;font-size:large;}
.conpane {position:absolute;right:0.5em;top:0;margin:0;padding:0;}
</style>
</head>
<body>
<form action="taw.cgi" method="post">
<div class="conpane">
<input type="button" onclick="document.location='<TMPL_VAR name=url>';" value ="Reload">
<input type="submit" value="Save" name="save" id="save">
<TMPL_IF name=password>
<input type="password" name="input_password" size="8" value="<TMPL_VAR name=input_password>">
</TMPL_IF>
<input type="hidden" name="mode" value="write">
</div>
<textarea name="text" id="text" class="edit"><TMPL_VAR name=cont></textarea>
</form>
</body>
</html>
